#include <gsl/gsl_linalg.h>

int callSVD_square(double* matrix, double *eigenvals, double *eigenvects, const int N)
{
   // Do the SVD on the square matrix: A = USV_T
   // 
   // matrix        I / NxN matrix A to be factored
   //               O / matrix U
   // eigenvalues   I / Empty array allocated to size N
   //               O / The singular values S, variances on principal axes.
   // eigenvects    I / Empty array of size NxN
   //               O / The matrix V, the principal axes, NOT transposed.
   
   int status=0;
   gsl_set_error_handler_off();
   gsl_vector *workspace = gsl_vector_alloc(N);
   gsl_vector *S = gsl_vector_alloc(N);
   gsl_matrix *A = gsl_matrix_alloc(N,N);
   gsl_matrix *V = gsl_matrix_alloc(N,N);
   
   for (int i=0; i<N; ++i)
      for (int j=0; j<N; ++j)
         gsl_matrix_set(A, i, j, matrix[i*N+j]);
   
   status = gsl_linalg_SV_decomp(A, V, S, workspace);
   
   for (int i=0; i<N; ++i)
      for (int j=0; j<N; ++j)
      {
         matrix[i*N+j] = gsl_matrix_get(A, i, j);
         eigenvects[i*N+j] = gsl_matrix_get(V, i, j);
      }
   
   for (int i=0; i<N; ++i)
      eigenvals[i] = gsl_vector_get(S, i);
      
   gsl_vector_free(workspace);
   gsl_vector_free(S);
   gsl_matrix_free(A);
   gsl_matrix_free(V);
   gsl_set_error_handler(0);
      
   return status;
}
